@@ -91,6 +91,7 @@ CONSTANT_MODULE_SUFFIX = '_Constants'
 
 SERVICE_REQUEST_MESSAGE_SUFFIX = '_Request'
 SERVICE_RESPONSE_MESSAGE_SUFFIX = '_Response'
+SERVICE_EVENT_MESSAGE_SUFFIX = '_Event'
 
 ACTION_GOAL_SUFFIX = '_Goal'
 ACTION_RESULT_SUFFIX = '_Result'
@@ -436,6 +437,21 @@ class Annotatable:
         """
         return [a.value for a in self.annotations if a.name == name]
 
+    def get_comment_lines(self):
+        """
+        Get the comment lines of the annotatable.
+
+        :returns: a list of comment lines
+        """
+        comments = [
+            x['text'] for x in self.get_annotation_values('verbatim') if
+            'language' in x and 'text' in x and x['language'] == 'comment'
+        ]
+        lines = []
+        for comment in comments:
+            lines.extend(comment.splitlines())
+        return lines
+
     def has_annotation(self, name):
         """
         Check if there is exactly one annotation of a specific type.
@@ -547,7 +563,7 @@ class Message:
 class Service:
     """A namespaced type containing a request and response message."""
 
-    __slots__ = ('namespaced_type', 'request_message', 'response_message')
+    __slots__ = ('namespaced_type', 'request_message', 'response_message', 'event_message')
 
     def __init__(
         self, namespaced_type: NamespacedType, request: Message,
@@ -580,6 +596,20 @@ class Service:
             namespaced_type.name + SERVICE_RESPONSE_MESSAGE_SUFFIX
         self.response_message = response
 
+        self.event_message = Message(
+            Structure(
+                NamespacedType(
+                    namespaces=namespaced_type.namespaces,
+                    name=f'{namespaced_type.name}{SERVICE_EVENT_MESSAGE_SUFFIX}'
+                ),
+                members=[
+                    Member(NamespacedType(['service_msgs', 'msg'], 'ServiceEventInfo'), 'info'),
+                    Member(BoundedSequence(request.structure.namespaced_type, 1), 'request'),
+                    Member(BoundedSequence(response.structure.namespaced_type, 1), 'response'),
+                ]
+            )
+        )
+
 
 class Action:
     """A namespaced type of an action including the derived types."""
